/*!
 * Should
 * Copyright(c) 2010-2014 TJ Holowaychuk <tj@vision-media.ca>
 * MIT Licensed
 */

var _eql = require('../eql');

var config = require('../config');

var util = require('../util');

module.exports = function(should, Assertion) {

  /**
   * Deep object equality comparison. For full spec see [`should-equal tests`](https://github.com/shouldjs/equal/blob/master/test.js).
   *
   * @name eql
   * @memberOf Assertion
   * @category assertion equality
   * @param {*} val Expected value
   * @param {string} [description] Optional message
   * @example
   *
   * (10).should.be.eql(10);
   * ('10').should.not.be.eql(10);
   * (-0).should.not.be.eql(+0);
   *
   * NaN.should.be.eql(NaN);
   *
   * ({ a: 10}).should.be.eql({ a: 10 });
   * [ 'a' ].should.not.be.eql({ '0': 'a' });
   */
  Assertion.add('eql', function(val, description) {
    var eql = _eql();
    this.params = {operator: 'to equal', expected: val, showDiff: true, message: description};

    var strictResult = eql(this.obj, val, should.config);

    var result = config.useOldDeepEqual ? strictResult: strictResult.result;

    if(!config.useOldDeepEqual && !strictResult.result) {
      this.params.details = util.formatEqlResult(strictResult, this.obj, val, should.format);
    }

    this.assert(result);
  });

  /**
   * Exact comparison using ===.
   *
   * @name equal
   * @memberOf Assertion
   * @category assertion equality
   * @alias Assertion#exactly
   * @param {*} val Expected value
   * @param {string} [description] Optional message
   * @example
   *
   * 10.should.be.equal(10);
   * 'a'.should.be.exactly('a');
   *
   * should(null).be.exactly(null);
   */
  Assertion.add('equal', function(val, description) {
    this.params = {operator: 'to be', expected: val, showDiff: true, message: description};

    this.assert(val === this.obj);
  });

  Assertion.alias('equal', 'exactly');
};